@@ -9,6 +9,7 @@
 #include "Geometry.h"
 #include "QtCV.h"
 #include "DebugUtils.h"
+#include "ImageMetaData.h"
 
 #include <KWindowSystem>
 
@@ -39,24 +40,6 @@ static const QString s_screenShotService = u"org.kde.KWin.ScreenShot2"_s;
 static const QString s_screenShotObjectPath = u"/org/kde/KWin/ScreenShot2"_s;
 static const QString s_screenShotInterface = u"org.kde.KWin.ScreenShot2"_s;
 
-static const QString s_logicalXKey = u"logicalX"_s;
-static const QString s_logicalYKey = u"logicalY"_s;
-
-// QImage::offset is a QPoint, so if we want to keep floating point precision when converting to
-// logical (wayland style) coordinates, we need to either make a new class to bundle a point or
-// store it a different way. We're using QImage::text to get the point, which is a bit weird,
-// but shouldn't require as much code as making a new class.
-inline static void setQImageLogicalXY(QImage &image, qreal x, qreal y)
-{
-    image.setText(s_logicalXKey, QString::number(x));
-    image.setText(s_logicalYKey, QString::number(y));
-}
-
-inline static QPointF qImageLogicalPointF(const QImage &image)
-{
-    return {image.text(s_logicalXKey).toDouble(), image.text(s_logicalYKey).toDouble()};
-}
-
 static QVariantMap screenShotFlagsToVardict(ImagePlatformKWin::ScreenShotFlags flags)
 {
     QVariantMap options;
@@ -88,9 +71,13 @@ QImage combinedImage(const QList<QImage> &images)
     }
     QRectF imageRect;
     qreal maxDpr = 0;
+    ImageMetaData::SubGeometryList geometryList;
     for (auto &i : images) {
-        maxDpr = std::max(maxDpr, i.devicePixelRatio());
-        imageRect |= QRectF{qImageLogicalPointF(i), i.deviceIndependentSize()};
+        const auto dpr = i.devicePixelRatio();
+        const auto rect = QRectF{ImageMetaData::logicalXY(i), i.deviceIndependentSize()};
+        maxDpr = std::max(maxDpr, dpr);
+        imageRect |= rect;
+        geometryList << ImageMetaData::subGeometryPropertyMap(rect, dpr);
     }
     static const auto finalFormat = QImage::Format_RGBA8888_Premultiplied;
     const bool allSameDpr = std::all_of(images.cbegin(), images.cend(), [maxDpr](const QImage &i){
@@ -102,7 +89,7 @@ QImage combinedImage(const QList<QImage> &images)
         for (auto &image : images) {
             // Explicitly setting the position and size so that you don't need to read
             // QPainter source code to understand how this works.
-            painter.drawImage({qImageLogicalPointF(image) * maxDpr, image.size()}, image);
+            painter.drawImage({ImageMetaData::logicalXY(image) * maxDpr, image.size()}, image);
         }
         painter.end();
         // Setting DPR after painting prevents it from affecting the coordinates of the QPainter.
@@ -110,6 +97,7 @@ QImage combinedImage(const QList<QImage> &images)
         // automatic scaling would occasionally use the wrong target position. I have no idea why.
         // It might not even be directly related to QPainter, so keep an eye out for bugs like that.
         finalImage.setDevicePixelRatio(maxDpr);
+        ImageMetaData::setSubGeometryList(finalImage, geometryList);
         return finalImage;
     }
     // We ceil to the next integer size up so that integer DPR images are always crisp.
@@ -124,7 +112,7 @@ QImage combinedImage(const QList<QImage> &images)
         auto rgbaImage = image.format() == finalImage.format() ? image : image.convertedTo(finalFormat);
         const auto mat = QtCV::qImageToMat(rgbaImage);
         // Region Of Interest to put the image in the main image.
-        const auto pos = qImageLogicalPointF(rgbaImage) * finalDpr;
+        const auto pos = ImageMetaData::logicalXY(rgbaImage) * finalDpr;
         const auto size = rgbaImage.deviceIndependentSize() * finalDpr;
         // Truncate to ints instead of rounding to prevent ROI from going out of bounds.
         const cv::Rect rect(pos.x(), pos.y(), size.width(), size.height());
@@ -135,6 +123,7 @@ QImage combinedImage(const QList<QImage> &images)
         cv::resize(mat, mainMat(rect), rect.size(), 0, 0, interpolation);
     }
     finalImage.setDevicePixelRatio(finalDpr);
+    ImageMetaData::setSubGeometryList(finalImage, geometryList);
     return finalImage;
 }
 
@@ -211,26 +200,22 @@ static ResultVariant readImage(int fileDescriptor, const QVariantMap &metadata)
         message.setArguments({windowId});
         const QDBusReply<QVariantMap> reply = QDBusConnection::sessionBus().call(message);
         if (reply.isValid()) {
-            const auto &windowTitle = reply.value().value(u"caption"_s).toString();
-            if (!windowTitle.isEmpty()) {
-                resultImage.setText(u"windowTitle"_s, windowTitle);
-                ExportManager::instance()->setWindowTitle(windowTitle);
-            }
+            ImageMetaData::setWindowTitle(resultImage, reply.value().value(u"caption"_s).toString());
             auto logicalX = Geometry::mapFromPlatformValue(reply.value().value(u"x"_s).toReal(), scale);
             auto logicalY = Geometry::mapFromPlatformValue(reply.value().value(u"y"_s).toReal(), scale);
-            setQImageLogicalXY(resultImage, logicalX, logicalY);
+            ImageMetaData::setLogicalXY(resultImage, logicalX, logicalY);
         }
     }
 
     const auto screenId = metadata.value(u"screen"_s).toString();
     if (!screenId.isEmpty()) {
-        resultImage.setText(u"screen"_s, screenId);
+        ImageMetaData::setScreen(resultImage, screenId);
         if (windowId.isEmpty()) {
             const auto screens = qGuiApp->screens();
             for (auto screen : screens) {
                 if (screen->name() == screenId) {
                     auto logicalPos = Geometry::mapFromPlatformPoint(screen->geometry().topLeft(), scale);
-                    setQImageLogicalXY(resultImage, logicalPos.x(), logicalPos.y());
+                    ImageMetaData::setLogicalXY(resultImage, logicalPos.x(), logicalPos.y());
                     break;
                 }
             }
